import React from 'react'
import fetch from 'isomorphic-fetch'
import ReactDOM from 'react-dom'
import url from 'url'

import formatOptions from './utils/formatOptions.js'
import LoginModal from './components/LoginModal'
import serialize from './utils/serializeQuery.js'
import renderErrWrapper from './components/errWrapper'

export const serializeQuery = serialize

let lastFetch = {
  count: 0
}

let loginModal = null
function getLoginModal () {
  if (!loginModal) {
    let container = document.createElement('div')
    document.body.appendChild(container)
    loginModal = ReactDOM.render(<LoginModal />, container)
  }
  return loginModal
}

function universalFetchProcess (res) {
  const { status } = res
  const rejectObject = {
    status,
    message: 'universe fetch process end'
  }

  // 306 页面跳转
  if (status === 306) {
    window.location.href = res.headers.get('location')

    rejectObject.type = 'redirect'
    return Promise.reject(rejectObject)
    // 401 无权限，进行登录操作
  } else if (status === 401) {
    let loginModal = getLoginModal()
    loginModal.show(lastFetch)
    loginModal.renderScanCode()

    rejectObject.type = 'noPrivilege'
    return Promise.reject(rejectObject)
    // 其他非200错误进行展示处理，兼容json及非json(500)
  } else if (status < 200 || status > 299) {
    res.text().then(errorBody => {
      let message = errorBody
      try {
        let json = JSON.parse(errorBody)
        message = json.message || JSON.stringify(json, null, 4)
      } catch (e) { }
      renderErrWrapper(message)
    })

    rejectObject.type = 'errorMessage'
    return Promise.reject(rejectObject)
  } else {
    // 200 系列
    return res.json()
  }
}

function processCtoken (url) {
  let tokenEl = document.getElementsByTagName('meta')['_csrf']
  if (tokenEl) {
    let token = tokenEl.content
    if (~url.indexOf('?')) {
      url += '&_csrf='
    } else {
      url += '?_csrf='
    }
    url += token
  }
  return url
}

// 只记录 /api 前缀的请求，这些需要登录
function memoryFetch (newUrl, ops) {
  const timeNow = Date.now()

  let parsedUrl = url.parse(newUrl)

  // 黑名单域名，不会进行记忆
  if (parsedUrl.host && parsedUrl.host.indexOf('bbs.cathay-ins.com.cn') >= 0) {
    return
  }
  if (parsedUrl.path.indexOf('/api') === 0) {
    if (lastFetch.time) {
      lastFetch.intervalTime = timeNow - lastFetch.time
    }
    lastFetch.time = timeNow
    lastFetch.url = newUrl
    lastFetch.options = ops
    lastFetch.count++
  }
}

export default function (rawUrl, options) {
  const { url, ops } = formatOptions(rawUrl, options)
  const newUrl = processCtoken(url)

  memoryFetch(newUrl, ops)

  return fetch(newUrl, ops)
    .then(universalFetchProcess)
}

export { LoginModal }
