var path = require('path')
var webpack = require('webpack')
var HtmlWebpackPlugin = require('html-webpack-plugin')
var eslintFriendlyFormatter = require('eslint-friendly-formatter')
var OpenBrowserPlugin = require('open-browser-webpack-plugin')
// var childProcess = require('child_process')

var dirname = path.resolve(__dirname)
var mockPort = require('./mock/config.js').port
var devPort = 3011

// childProcess.exec('nodemon ./mock/server.js')

module.exports = {
  entry: [
    'babel-polyfill',
    'react-hot-loader/patch', // activate HMR for React
    'webpack-dev-server/client?http://localhost:' + devPort, // bundle the client for webpack-dev-server and connect to the provided endpoint
    'webpack/hot/only-dev-server', // bundle the client for hot reloading,only- means to only hot reload for successful updates
    './examples/index.js'
  ],
  output: {
    filename: 'bundle.[hash].js',
    path: dirname + '/lib',
    publicPath: '/' // necessary for HMR to know where to load the hot update chunks
  },
  devtool: 'inline-source-map',
  module: {
    rules: [
      {
        test: /\.css$/,
        use: [
          'style-loader',
          'css-loader'
        ]
      },
      {
        test: /\.jsx?$/,
        exclude: /node_modules/,
        use: [
          'babel-loader',
          {
            loader: 'eslint-loader',
            options: {
              formatter: eslintFriendlyFormatter
            }
          }
        ]
      },
      {
        test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
        loader: 'url-loader',
        query: {
          limit: 10000,  // 10kb
          name: '[name].[hash:7].[ext]'
        }
      },
      {
        test: /\.(woff2?|eot|ttf|otf)(\?.*)?$/,
        loader: 'url-loader',
        query: {
          limit: 10000,
          name: '[name].[hash:7].[ext]'
        }
      }
    ]
  },
  resolve: {
    extensions: ['.js', '.jsx', '.json']
  },
  plugins: [
    new webpack.HotModuleReplacementPlugin(), // enable HMR globally
    new webpack.NamedModulesPlugin(), // prints more readable module names in the browser console on HMR updates
    new webpack.NoEmitOnErrorsPlugin(),  // do not emit compiled assets that include errors
    new webpack.ProgressPlugin(function (percent, msg) {
      percent = Math.floor(percent * 1000) / 10
      percent += '%'
      while (percent.length < 10) {
        percent += ' '
      }
      process.stdout.write(percent + msg + '\r')
    }),
    new HtmlWebpackPlugin({
      template: dirname + '/examples/index.html',
      hash: false,
      filename: 'index.html',
      inject: 'body',
      minify: {
        collapseWhitespace: true
      }
    }),
    new OpenBrowserPlugin({
      url: 'http://localhost:' + devPort
    })
  ],
  devServer: {
    host: 'localhost',
    port: devPort,
    disableHostCheck: true,
    historyApiFallback: true, // respond to 404s with index.html
    hot: true, // enable HMR on the server
    proxy: {
      '/**': {
        target: 'http://localhost:' + mockPort
      }
    }
  }
}
